@@ -1,6 +1,6 @@
 /********************************************
 cast.c
-copyright 2009-2012,2013 Thomas E. Dickey
+copyright 2009-2021,2024, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,65 +11,20 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: cast.c,v 1.19 2013/12/26 23:23:55 tom Exp $
- * @Log: cast.c,v @
- * Revision 1.6  1996/08/11 22:07:50  mike
- * Fix small bozo in rt_error("overflow converting ...")
- *
- * Revision 1.5  1995/06/18  19:17:45  mike
- * Create a type Int which on most machines is an int, but on machines
- * with 16bit ints, i.e., the PC is a long.  This fixes implicit assumption
- * that int==long.
- *
- * Revision 1.4  1995/06/06  00:02:02  mike
- * fix cast in d_to_l()
- *
- * Revision 1.3  1993/07/17  13:22:45  mike
- * indent and general code cleanup
- *
- * Revision 1.2	 1993/07/04  12:51:41  mike
- * start on autoconfig changes
- *
- * Revision 5.5	 1993/03/06  18:49:45  mike
- * rm rt_overflow from check_strnum
- *
- * Revision 5.4	 1993/02/13  21:57:20  mike
- * merge patch3
- *
- * Revision 5.3.1.4  1993/01/22	 15:05:19  mike
- * pow2->mpow2 for linux
- *
- * Revision 5.3.1.3  1993/01/22	 14:18:33  mike
- * const for strtod and ansi picky compilers
- *
- * Revision 5.3.1.2  1993/01/20	 12:53:06  mike
- * d_to_l()
- *
- * Revision 5.3.1.1  1993/01/15	 03:33:37  mike
- * patch3: safer double to int conversion
- *
- * Revision 5.3	 1992/11/28  23:48:42  mike
- * For internal conversion numeric->string, when testing
- * if integer, use longs instead of ints so 16 and 32 bit
- * systems behave the same
- *
- * Revision 5.2	 1992/08/17  14:19:45  brennan
- * patch2: After parsing, only bi_sprintf() uses string_buff.
- *
- * Revision 5.1	 1991/12/05  07:55:41  brennan
- * 1.1 pre-release
- *
-*/
+ * $MawkId: cast.c,v 1.30 2024/08/25 17:09:56 tom Exp $
+ */
 
-/*  cast.c  */
+#define Visible_CELL
+#define Visible_RE_DATA
+#define Visible_STRING
 
-#include "mawk.h"
-#include "field.h"
-#include "memory.h"
-#include "scan.h"
-#include "repl.h"
+#include <mawk.h>
+#include <field.h>
+#include <memory.h>
+#include <scan.h>
+#include <repl.h>
 
-int mpow2[NUM_CELL_TYPES] =
+const int mpow2[NUM_CELL_TYPES] =
 {1, 2, 4, 8, 16, 32, 64, 128, 256, 512};
 
 void
@@ -88,8 +43,8 @@ cast1_to_d(CELL *cp)
 	{
 	    register STRING *s = (STRING *) cp->ptr;
 
-#ifdef FPE_TRAPS_ON		/* look for overflow error */
 	    errno = 0;
+#ifdef FPE_TRAPS_ON		/* look for overflow error */
 	    cp->dval = strtod(s->str, (char **) 0);
 	    if (errno && cp->dval != 0.0)	/* ignore underflow */
 		rt_error("overflow converting %s to double", s->str);
@@ -131,8 +86,8 @@ cast2_to_d(CELL *cp)
     case C_STRING:
 	s = (STRING *) cp->ptr;
 
-#ifdef FPE_TRAPS_ON		/* look for overflow error */
 	errno = 0;
+#ifdef FPE_TRAPS_ON		/* look for overflow error */
 	cp->dval = strtod(s->str, (char **) 0);
 	if (errno && cp->dval != 0.0)	/* ignore underflow */
 	    rt_error("overflow converting %s to double", s->str);
@@ -165,8 +120,8 @@ cast2_to_d(CELL *cp)
     case C_STRING:
 	s = (STRING *) cp->ptr;
 
-#ifdef FPE_TRAPS_ON		/* look for overflow error */
 	errno = 0;
+#ifdef FPE_TRAPS_ON		/* look for overflow error */
 	cp->dval = strtod(s->str, (char **) 0);
 	if (errno && cp->dval != 0.0)	/* ignore underflow */
 	    rt_error("overflow converting %s to double", s->str);
@@ -182,10 +137,26 @@ cast2_to_d(CELL *cp)
     cp->type = C_DOUBLE;
 }
 
+#define DoubleToString(target,source) \
+	if (source->dval >= (double) Max_Long) { \
+	    ULong lval = d_to_UL(source->dval); \
+	    if (lval == source->dval) { \
+		sprintf(target, ULONG_FMT, lval); \
+	    } else { \
+		sprintf(target, string(CONVFMT)->str, source->dval); \
+	    } \
+	} else { \
+	    Long lval = d_to_L(source->dval); \
+	    if (lval == source->dval) { \
+		sprintf(target, LONG_FMT, lval); \
+	    } else { \
+		sprintf(target, string(CONVFMT)->str, source->dval); \
+	    } \
+	}
+
 void
 cast1_to_s(CELL *cp)
 {
-    register Int lval;
     char xbuff[260];
 
     switch (cp->type) {
@@ -196,12 +167,7 @@ cast1_to_s(CELL *cp)
 
     case C_DOUBLE:
 
-	lval = d_to_I(cp->dval);
-	if (lval == cp->dval)
-	    sprintf(xbuff, INT_FMT, lval);
-	else
-	    sprintf(xbuff, string(CONVFMT)->str, cp->dval);
-
+	DoubleToString(xbuff, cp);
 	cp->ptr = (PTR) new_STRING(xbuff);
 	break;
 
@@ -221,7 +187,6 @@ cast1_to_s(CELL *cp)
 void
 cast2_to_s(CELL *cp)
 {
-    register Int lval;
     char xbuff[260];
 
     switch (cp->type) {
@@ -232,12 +197,7 @@ cast2_to_s(CELL *cp)
 
     case C_DOUBLE:
 
-	lval = d_to_I(cp->dval);
-	if (lval == cp->dval)
-	    sprintf(xbuff, INT_FMT, lval);
-	else
-	    sprintf(xbuff, string(CONVFMT)->str, cp->dval);
-
+	DoubleToString(xbuff, cp);
 	cp->ptr = (PTR) new_STRING(xbuff);
 	break;
 
@@ -264,12 +224,7 @@ cast2_to_s(CELL *cp)
 
     case C_DOUBLE:
 
-	lval = d_to_I(cp->dval);
-	if (lval == cp->dval)
-	    sprintf(xbuff, INT_FMT, lval);
-	else
-	    sprintf(xbuff, string(CONVFMT)->str, cp->dval);
-
+	DoubleToString(xbuff, cp);
 	cp->ptr = (PTR) new_STRING(xbuff);
 	break;
 
@@ -305,15 +260,18 @@ cast_to_RE(CELL *cp)
 void
 cast_for_split(CELL *cp)
 {
-    static char meta[] = "^$.*+?|[]()";
-    static char xbuff[] = "\\X";
-    int c;
+#ifndef NO_INTERVAL_EXPR
+    static const char meta[] = "^$.*+?|[](){}";
+#else
+    static const char meta[] = "^$.*+?|[]()";
+#endif
     size_t len;
 
     if (cp->type < C_STRING)
 	cast1_to_s(cp);
 
     if ((len = string(cp)->len) == 1) {
+	int c;
 	if ((c = string(cp)->str[0]) == ' ') {
 	    free_STRING(string(cp));
 	    cp->type = C_SPACE;
@@ -338,6 +296,7 @@ cast_for_split(CELL *cp)
 	    return;
 #endif
 	} else if ((strchr) (meta, c)) {
+	    static char xbuff[] = "\\X";
 	    xbuff[1] = (char) c;
 	    free_STRING(string(cp));
 	    cp->ptr = (PTR) new_STRING(xbuff);
@@ -382,8 +341,8 @@ check_strnum(CELL *cp)
     case SC_MINUS:
     case SC_DOT:
 
-#ifdef FPE_TRAPS_ON
 	errno = 0;
+#ifdef FPE_TRAPS_ON
 	cp->dval = strtod((char *) s, &temp);
 	/* make overflow pure string */
 	if (errno && cp->dval != 0.0)
@@ -429,27 +388,58 @@ cast_to_REPL(CELL *cp)
 Int
 d_to_I(double d)
 {
-    if (d >= Max_Int)
-	return Max_Int;
-    if (d > -Max_Int)
-	return (Int) d;
-    return -Max_Int;
+    Int result;
+
+    if (d >= (double) Max_Int) {
+	result = Max_Int;
+    } else if (d < 0) {
+	if (-d <= (double) Max_Int) {
+	    result = (Int) d;
+	} else {
+	    result = -Max_Int;
+	}
+    } else {
+	result = (Int) d;
+    }
+    return result;
 }
 
-/* convert a double to UInt (this is not as simple as a
-   cast because the results are undefined if it won't fit).
-   Truncate large values to Max_UInt or 0
-   Send nans to 0
-*/
-
-UInt
-d_to_U(double d)
+Long
+d_to_L(double d)
 {
-    if (d >= Max_UInt)
-	return Max_UInt;
-    if (d > 0)
-	return (UInt) d;
-    return 0;
+    Long result;
+
+    if (d >= (double) Max_Long) {
+	result = Max_Long;
+    } else if (d < 0) {
+	if (-d <= (double) Max_Long) {
+	    result = (Long) d;
+	} else {
+	    result = -Max_Long;
+	}
+    } else {
+	result = (Long) d;
+    }
+    return result;
+}
+
+ULong
+d_to_UL(double d)
+{
+    ULong result;
+
+    if (d >= (double) Max_ULong) {
+	result = Max_ULong;
+    } else if (d < 0) {
+	if (-d < (double) Max_ULong) {
+	    result = ((Max_ULong + (ULong) d) + 1);
+	} else {
+	    result = -Max_ULong;
+	}
+    } else {
+	result = (ULong) d;
+    }
+    return result;
 }
 
 #ifdef NO_LEAKS
