@@ -1,6 +1,6 @@
 /********************************************
 code.c
-copyright 2009-2019,2023, Thomas E. Dickey
+copyright 2009-2023,2024, Thomas E. Dickey
 copyright 1991-1994,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,18 +11,23 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: code.c,v 1.41 2023/08/01 07:42:59 tom Exp $
+ * $MawkId: code.c,v 1.47 2024/08/25 19:47:39 tom Exp $
  */
 
-#include "mawk.h"
-#include "code.h"
-#include "init.h"
-#include "jmp.h"
-#include "field.h"
+#define Visible_CELL
+#define Visible_CODEBLOCK
+#define Visible_FBLOCK
+#define Visible_STRING
+
+#include <mawk.h>
+#include <code.h>
+#include <init.h>
+#include <jmp.h>
+#include <field.h>
 
 #ifdef NO_LEAKS
-#include "repl.h"
-#include "scan.h"
+#include <repl.h>
+#include <scan.h>
 #endif
 
 static CODEBLOCK *new_code(void);
@@ -92,6 +97,22 @@ xcode2(int op, PTR ptr)
     code_ptr = p;
 }
 
+/* code an op and a function-pointer in the active_code */
+void
+xfunc2(int op, PF_CP fnc)
+{
+    register INST *p = code_ptr + 2;
+
+    if (p >= code_warn) {
+	code_grow();
+	p = code_ptr + 2;
+    }
+
+    p[-2].op = op;
+    p[-1].fnc = fnc;
+    code_ptr = p;
+}
+
 /* code two ops in the active_code */
 void
 code2op(int x, int y)
@@ -210,12 +231,20 @@ be_setup(int scope)
     *main_code_p = active_code;
 
     if (scope == SCOPE_BEGIN) {
-	if (!begin_code_p)
+	if (!begin_code_p) {
+	    TRACE(("be_setup: BEGIN\n"));
 	    begin_code_p = new_code();
+	} else {
+	    TRACE(("be_setup: BEGIN (again)\n"));
+	}
 	active_code = *begin_code_p;
     } else {
-	if (!end_code_p)
+	if (!end_code_p) {
+	    TRACE(("be_setup: END\n"));
 	    end_code_p = new_code();
+	} else {
+	    TRACE(("be_setup: END (again)\n"));
+	}
 	active_code = *end_code_p;
     }
 }
@@ -266,7 +295,9 @@ free_codes(const char *tag, INST * base, size_t size)
 
     if (base != 0 && size != 0) {
 	for (cdp = base; cdp < last; ++cdp) {
+#ifndef NO_LEAKS
 	    TRACE_INST(cdp, base);
+#endif
 
 	    switch ((MAWK_OPCODES) (cdp->op)) {
 	    case AE_PUSHA:
@@ -314,6 +345,7 @@ free_codes(const char *tag, INST * base, size_t size)
 	    case _RANGE:
 		cdp += 4;	/* PAT1 */
 		break;
+	    case _CALLX:
 	    case _CALL:
 		cdp += 1 + cdp[2].op;
 		break;
